/***************************************************************************
                               caliasfilter.cpp
                             -------------------
    begin                : Mon Jul 29 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qstring.h>
#include <qdom.h>
 
// Kmud includes
#include <kmuddebug.h>
#include <kmudapi.h>
 
// local includes
#include "caliasfilter.h"
#include "caliasgroup.h"

///////////////////////////////////////////////////////////////////////////////

CAliasFilter::CAliasFilter(KmudAPI* api) : QObject(0, "caliasfilter"), KmudFilter(),
	m_kmudAPI(api), m_resolver(0)
{
	setName("AliasFilter");
	if (m_kmudAPI != 0)
	{
		QObject::connect(m_kmudAPI, SIGNAL(profileChanged()),
		                 this, SLOT(slotProfileChanged()));
		QObject::connect(m_kmudAPI->profileAPI(), SIGNAL(profileDataChanged(const QString&, uint)),
		                 this, SLOT(slotProfileDataChanged(const QString&, uint)));
	}
}

///////////////////////////////////////////////////////////////////////////////

CAliasFilter::~CAliasFilter()
{
	if (m_resolver != 0)
		delete m_resolver;
}

///////////////////////////////////////////////////////////////////////////////

QString CAliasFilter::filterInput(QString& input)
{
	kdDebug(DAREA_CORE) << "AliasFilter::filterInput\n";
	
	if (m_resolver == 0)
		return input;

	QStringList lines = QStringList::split('\n', input, true);
	uint resolvCount = 0;
	for (uint i = 0; i < lines.count(); ++i)
	{
		if (!lines[i].isEmpty())
		{
			while (true)
			{
				QString resolved = m_resolver->resolve(lines[i], ";");
				if (resolved == lines[i])
					break;
				lines[i] = resolved;
				resolvCount++;
			}
		}
	}

	kdDebug(DAREA_CORE) << resolvCount << " resolver iterations\n";
	return lines.join("\n");
}

///////////////////////////////////////////////////////////////////////////////

void CAliasFilter::slotProfileChanged()
{
	kdDebug(DAREA_CORE) << "AliasFilter::slotProfileChanged\n";
	if (m_kmudAPI == 0)
		return;

	if (m_resolver != 0)
	{
		delete m_resolver;
		m_resolver = 0;
	}

	m_profile = m_kmudAPI->profile();
	kdDebug(DAREA_CORE) << "profile: " << m_profile << endl;
	
	if (m_profile.isEmpty())
		return;

	QStringList parts = QStringList::split('/', m_profile);

	QDomElement mudProfile = m_kmudAPI->profileAPI()->openProfile(parts[0]);
	if (mudProfile.isNull())
	{
		kdDebug(DAREA_CORE) << "mud profile element is null\n";
		return;
	}

	QDomNode aliasNode = mudProfile.namedItem("aliasgroup");
	if (aliasNode.isNull() || !aliasNode.isElement())
	{
		kdDebug(DAREA_CORE) << "mud alias element is null\n";
		return;
	}

	QDomElement aliasElement = aliasNode.toElement();

	CAliasGroup* masterGroup = new CAliasGroup("MasterGroup");
	masterGroup->setEnabled(true);
	m_resolver = masterGroup;
	CAliasGroup mudAlias;
	
	if (mudAlias.readFromDOM(aliasElement))
	{
		mudAlias.setName("MudAliases");
		masterGroup->insert(mudAlias);
	}
	else
	{
		kdDebug(DAREA_CORE) << "failed to load mud aliases\n";
	}

	
    if (parts.count() == 1)
    	return;
     
	QDomElement charProfile = m_kmudAPI->profileAPI()->openProfile(m_profile);
	if (charProfile.isNull())
	{
		kdDebug(DAREA_CORE) << "char profile element is null\n";
		return;
    }
    
	aliasNode = charProfile.namedItem("aliasgroup");
	if (aliasNode.isNull() || !aliasNode.isElement())
	{
		kdDebug(DAREA_CORE) << "char alias element is null\n";
		return;
	}
    
	aliasElement = aliasNode.toElement();

	CAliasGroup charAlias;

	if (charAlias.readFromDOM(aliasElement))
	{
		charAlias.setName("CharAliases");
		masterGroup->insert(charAlias, 0);
	}
	else
	{
		kdDebug(DAREA_CORE) << "failed to load char aliases\n";
	}
}

///////////////////////////////////////////////////////////////////////////////

void CAliasFilter::slotProfileDataChanged(const QString& profile, uint type)
{
	kdDebug(DAREA_CORE) << "AliasFilter::slotProfileDataChanged(" << profile << ", " << type << ")\n";
	if (profile == m_profile && (type & ProfileAPI::ALIAS) != 0)
		slotProfileChanged();
}

// end of file
