/***************************************************************************
                               caliasgroup.cpp
                             -------------------
    begin                : Thu Jul 25 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qdom.h>
#include <qstring.h>
#include <qstringlist.h>

// local includes
#include "calias.h"
#include "caliasgroup.h"

///////////////////////////////////////////////////////////////////////////////

CAliasGroup::CAliasGroup() : CAliasItem()
{
	m_childList.setAutoDelete(true);
}

///////////////////////////////////////////////////////////////////////////////

CAliasGroup::CAliasGroup(const QString& name) : CAliasItem(name)
{
	m_childList.setAutoDelete(true);
}

///////////////////////////////////////////////////////////////////////////////

CAliasGroup::~CAliasGroup()
{
}

///////////////////////////////////////////////////////////////////////////////

bool CAliasGroup::insert(const CAliasItem& item, int index)
{
	if (index > (int)m_childList.count())
		return false;
		
	for (uint i = 0; i < m_childList.count(); ++i)
	{
		if (m_childList.at(i)->name() == item.name())
			return false;
	}

	CAliasItem* newItem = 0;
	if (item.type() == Alias)
	{
		newItem = new CAlias(*(static_cast<const CAlias*>(&item)));
	}
	else if (item.type() == Group)
	{
		newItem = new CAliasGroup(*(static_cast<const CAliasGroup*>(&item)));
	}
	else
	{
		qWarning("CAliasGroup::insert: unknown item type %d", item.type());
		return false; // should not be reached
	}

	if (newItem == 0)
		return false;

	if (index < 0 || index == (int)m_childList.count())
	{
		m_childList.append(newItem);
		return true;
	}

	bool ok = m_childList.insert((uint)index, newItem);
	if (!ok)
		delete newItem;

	return ok;
}

///////////////////////////////////////////////////////////////////////////////

bool CAliasGroup::remove(const CAliasItem& item)
{
	for (uint i = 0; i < m_childList.count(); ++i)
	{
		if (m_childList.at(i)->name() == item.name() &&
			m_childList.at(i)->type() == item.type())
		{
			return m_childList.remove();
		}
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////

bool CAliasGroup::readFromDOM(const QDomElement& dom)
{
	if (dom.isNull())
		return false;

	if (!CProfileItem::readFromDOM(dom))
		return false;

	for (QDomNode node = dom.firstChild(); !node.isNull(); node = node.nextSibling())
	{
		if (node.isElement())
		{
			QDomElement element = node.toElement();
			if (node.nodeName() == "alias")
			{
				CAlias alias;
				if (!alias.readFromDOM(element))
					return false;
				m_childList.append(new CAlias(alias));
			}
			else if (node.nodeName() == "aliasgroup")
			{
				CAliasGroup group;
				if (!group.readFromDOM(element))
					return false;
				m_childList.append(new CAliasGroup(group));
			}
		}
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////

bool CAliasGroup::writeToDOM(QDomElement& dom)
{
	if (dom.isNull())
		return false;

	if (!CProfileItem::writeToDOM(dom))
		return false;

	for (uint i = 0; i < m_childList.count(); ++i)
	{
		QDomElement domElement;
		if (m_childList.at(i)->type() == Alias)
			domElement = dom.ownerDocument().createElement("alias");
		else
			domElement = dom.ownerDocument().createElement("aliasgroup");
			
		if (!m_childList.at(i)->writeToDOM(domElement))
			return false;
		dom.appendChild(domElement);
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////

bool CAliasGroup::resolveCommands(QStringList& inputList)
{
	if (!m_enabled)
		return false;

	if (inputList.count() == 0)
		return false;

	bool cmdReplaced = false;
	for (uint cmdCount = 0; cmdCount < inputList.count(); ++cmdCount)
	{
		QStringList words = splitCommand(inputList[cmdCount]);
		if (words.count() > 0)
		{
			for (uint childCount = 0; childCount < m_childList.count(); ++childCount)
			{
				CAliasItem* item = m_childList.at(childCount);
				if (item->isEnabled())
				{
					if (item->type() == Alias)
					{
						CAlias* alias = static_cast<CAlias*>(item);
						if (alias->alias() == words[0])
						{
							inputList[cmdCount] = alias->replace(words);
							cmdReplaced = true;
							break;
						}
					}
					else
					{
						CAliasGroup* group = static_cast<CAliasGroup*>(item);
						QStringList replacement;
						replacement.append(inputList[cmdCount]);
						if (group->resolveCommands(replacement))
						{
							inputList[cmdCount] = replacement[0];
							cmdReplaced = true;
							break;
						}
					}
				}
			}
		}
	}

	return cmdReplaced;
}

// end of file
