/***************************************************************************
                               caliasgroup.h
                             -------------------
    begin                : Thu Jul 25 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CALIASGROUP_H
#define CALIASGROUP_H

// Qt includes
#include <qptrlist.h>

// local includes
#include "calias.h"

/** A class grouping aliias items and other alias group items
  * @author Kmud Developer Team
  */

class CAliasGroup : public CAliasItem {
public:

	/**
	  * Constructs a disabled and unnamed alias group.
	  */
	CAliasGroup();

	/**
	  * Constructs a disbaled alis group with the given name.
	  * @param name the item's name. Any string allowed.
	  */
	CAliasGroup(const QString& name);

	CAliasGroup(const CAliasGroup& group) : CAliasItem(group) {
		if (&group == this) return;
		copy(group);
	}

	virtual ~CAliasGroup();

	inline CAliasGroup& operator=(const CAliasGroup& group) {
		if (&group != this)
			copy(group);
		return *this;
	}

	inline Type type() const { return Group; }
	
	bool insert(const CAliasItem& alias, int index = -1);
	bool remove(const CAliasItem& alias);

	/**
	  * Read the item's content from the given QDomElement.
	  * @return true if reading succeeded, false otherwise.
	  */
	bool readFromDOM(const QDomElement& dom);

	/**
	  * Write the item's content to the given QDomElement.
	  * @return true if reading succeeded, false otherwise.
	  */
	bool writeToDOM(QDomElement& dom);

protected:
	QPtrList<CAliasItem> m_childList;

protected:
	bool resolveCommands(QStringList& inputList);

private:
	inline void copy(const CAliasGroup& other) {
		CAliasItem::operator=(other);
		m_childList.clear();
		QPtrList<CAliasItem> childList = other.m_childList;
		for (uint i = 0; i < childList.count(); ++i)
		{
			CAliasItem* item = childList.at(i);
			if (item != 0)
			{
				if (item->type() == Alias)
				{
					CAlias* alias = static_cast<CAlias*>(item);
					m_childList.append(new CAlias(*alias));
				}
				else if (item->type() == Group)
				{
					CAliasGroup* group = static_cast<CAliasGroup*>(item);
					m_childList.append(new CAliasGroup(*group));
				}
			}
		}
	}
};

#endif

// end of file
