/***************************************************************************
                               autologinfilter.cpp
                             -------------------
    begin                : Mon Mar 24 2003
    copyright            : (C) 2003 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cautologinfilter.h"

#include <kmudapi.h>
#include <kmuddebug.h>
#include <kmemconfig.h>

#include <domconfig.h>

CAutoLoginFilter::CAutoLoginFilter(KmudAPI *api,QObject* parent, const char* name) :QObject(parent,name)
{
	m_API = api;	
	m_mudProfile = QString::null;
	m_charProfile = QString::null;
	setFilterName("AutoLoginFilter");
	connect(m_API, SIGNAL(profileChanged()), this, SLOT(slotProfileChanged()));
	m_LoginHasBeenSent=true;
	m_PasswordHasBeenSent=true;

}

CAutoLoginFilter::~CAutoLoginFilter()
{
}

/** This method is called when output is sent to the mud */
QString CAutoLoginFilter::filterOutput(QString &s)
{
	kdDebug(DAREA_CORE) << "[" << s << "]" << endl;
	autoLogin(s);
	
	return s;	
}

void CAutoLoginFilter::slotProfileChanged()
{
	QString profileName = m_API->profile();

	if (profileName.isEmpty())
	{
		m_mudProfile = QString::null;
		m_charProfile = QString::null;
		m_LoginHasBeenSent=true;
		m_PasswordHasBeenSent=true;

	}
	else
	{
		QStringList names = QStringList::split("/", profileName);
		m_charProfile = profileName;
		m_mudProfile = names[0];
		m_LoginHasBeenSent=false;
		m_PasswordHasBeenSent=false;
	}

	kdDebug(DAREA_CORE) << "MUD PROFILE = " << m_mudProfile << endl;
	kdDebug(DAREA_CORE) << "CHAR PROFILE = " << m_charProfile << endl;
}

void CAutoLoginFilter::autoLogin(QString output)
{
	if (m_LoginHasBeenSent && m_PasswordHasBeenSent)
	{
		return;
	}

	QString charName = "";
	QString password = "";
	QString loginPrompt = "";
	QString passwordPrompt = "";

	QDomElement profileElement = m_API->profileAPI()->openProfile(m_charProfile);
	if (profileElement.isNull())
	{
		return ;
	}

	// find mapper config
	DomConfig* charConfig = DomConfig::findGroup("/", profileElement);
	if (charConfig == NULL)
	{
		return;
	}
	else
	{
		KMemConfig *config = charConfig->config();
		config->setGroup("Login");

		charName = config->readEntry("Login","").stripWhiteSpace();
		password = config->readEntry("Password","").stripWhiteSpace();
	}


	// try to open the profile
	profileElement = m_API->profileAPI()->openProfile(m_mudProfile);
	if (profileElement.isNull())
	{
		return ;
	}

	DomConfig* mudConfig = DomConfig::findGroup("/", profileElement);
	if (mudConfig == NULL)
	{
		return;
	}
	else
	{
		KMemConfig *config = mudConfig->config();
		loginPrompt = config->readEntry("LoginPrompt","").stripWhiteSpace();
		passwordPrompt = config->readEntry("PasswordPrompt","").stripWhiteSpace();
	}

	if (charName!="" && loginPrompt!="")
	{
		if (!m_LoginHasBeenSent)
		{
			if (output.contains(loginPrompt,false))
			{
				//FIXME_jp : The charName needs adding to the output
				m_API->sendCommand(charName+"\n",false);
			  	m_LoginHasBeenSent=true;
			}

		}

	}
	else
	{
		m_LoginHasBeenSent=true;
	}

	if (password!="" && passwordPrompt!="")
	{
		if (!m_PasswordHasBeenSent)
		{
			if (output.contains(passwordPrompt,false))
			{
				m_API->sendCommand(password+"\n",false);
				m_PasswordHasBeenSent=true;
			}
		}
	}
	else
	{
		m_PasswordHasBeenSent=true;
	}
}
