/***************************************************************************
                               cprofile.cpp
                             -------------------
    begin                : Sat Mar 2 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cprofile.h"

// Qt includes
#include <qfile.h>
#include <qtextstream.h>

// Kmud includes
#include <kmuddebug.h>
#include <domconfig.h>

///////////////////////////////////////////////////////////////////////////////

CProfile::CProfile()
{
}

///////////////////////////////////////////////////////////////////////////////

CProfile::~CProfile()
{
}

///////////////////////////////////////////////////////////////////////////////

void CProfile::create()
{
	kdDebug(DAREA_CORE_CONFIGMANAGER) << "CProfile::create\n";
	QDomDocument doc("Kmud-Profile");
	
	QDomElement profileElement = doc.createElement("profile");
	profileElement.setAttribute("version", "1.0");
	profileElement.setAttribute("type", "character");
	doc.appendChild(profileElement);
	
	m_profileElement = profileElement;
	// create top level config element if it doesn't exist
	DomConfig::findGroup("/", m_profileElement, true);
	m_doc = doc;
}

///////////////////////////////////////////////////////////////////////////////

bool CProfile::open(const QString& filename)
{
	if (!m_filename.isEmpty() || filename.isEmpty())
		return false;
	kdDebug(DAREA_CORE_CONFIGMANAGER) << "CProfile::open("<< filename <<")\n";
	
	QFile file(filename);
	
	if (!file.exists() || !file.open(IO_ReadOnly))
	{
		kdDebug(DAREA_CORE_CONFIGMANAGER) << "CProfile::open(" << filename << "): "
		    << "QFile::open failed\n";
		return false;
	}
	
	QDomDocument doc("Kmud-Profile");
	if (!doc.setContent(&file))
	{
		kdDebug(DAREA_CORE_CONFIGMANAGER) << "CProfile::open(" << filename << "): "
		    << "QDomDocument::setContent failed\n";
		file.close();
		return false;
	}
	file.close();
	
	QDomElement docElement = doc.documentElement();
	if (docElement.nodeName() != "profile")
	{
		kdDebug(DAREA_CORE_CONFIGMANAGER) << "CProfile::open(" << filename << "): "
		    << "document element is " << docElement.nodeName() << " and not profile\n";
		return false;
	}
	
	m_filename = filename;
	m_profileElement = docElement;
	// create top level config element if it doesn't exist
	DomConfig::findGroup("/", m_profileElement, true);
	
	m_doc = doc;

	return true;
}

///////////////////////////////////////////////////////////////////////////////

bool CProfile::save()
{
	if (m_filename.isEmpty())
		return false;
	else
		return saveAs(m_filename);
}

///////////////////////////////////////////////////////////////////////////////

bool CProfile::saveAs(const QString& filename)
{
	if (filename.isEmpty())
		return false;
	
	kdDebug(DAREA_CORE_CONFIGMANAGER) << "CProfile::saveAs("<< filename <<")\n";
	QFile file(filename);
	
	if (!file.open(IO_WriteOnly))
	{
		kdDebug(DAREA_CORE_CONFIGMANAGER) << "CProfile::saveAs(" << filename << "): "
		    << "QFile::open failed\n";
		return false;
	}

	QTextStream stream(&file);
	stream.setEncoding(QTextStream::UnicodeUTF8);
	QString content = m_doc.toString();
	if (!content.startsWith("<?xml"))
		stream << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
	stream << content;
	file.close();
	
	m_filename = filename;
	
	return true;
}

// end of file
