/***************************************************************************
                               cprofileitem.cpp
                             -------------------
    begin                : Thu Jul 25 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qdom.h>

// local includes
#include "cprofileitem.h"

///////////////////////////////////////////////////////////////////////////////

CProfileItem::CProfileItem(const QString& name)	: m_enabled(false), m_name(name)
{
}

///////////////////////////////////////////////////////////////////////////////

bool CProfileItem::readFromDOM(const QDomElement& dom)
{
	if (dom.isNull())
		return false;

	m_enabled = readBoolAttribute(dom, "enabled", false);

	QDomNode node = dom.namedItem("name");
	if (node.isNull() || !node.isElement())
	{
		m_name = QString::null;
	}
	else
	{
		QDomElement nametag = node.toElement();
		m_name = nametag.text();
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////

bool CProfileItem::writeToDOM(QDomElement& dom)
{
	if (dom.isNull())
		return false;

	writeAttribute(dom, "enabled", m_enabled);

	QDomElement nametag = dom.ownerDocument().createElement("name");
	QDomNode name = dom.ownerDocument().createTextNode(m_name);
	nametag.appendChild(name);
	dom.appendChild(nametag);

	return true;
}

///////////////////////////////////////////////////////////////////////////////

void CProfileItem::writeAttribute(QDomElement& element, const QString& name, bool value)
{
	element.setAttribute(name, (value ? "true" : "false"));
}

///////////////////////////////////////////////////////////////////////////////

bool CProfileItem::readBoolAttribute(const QDomElement& element, const QString& name, bool def)
{
	QString strValue = element.attribute(name);
	if (strValue.isEmpty())
		return def;
	strValue = strValue.lower();
	if (strValue == "true")
		return true;
	if (strValue == "false");
		return false;
	return def;
}

// end of file
