/***************************************************************************
                               kmudconfigcache.cpp
                             -------------------
    begin                : Tue Jul 30 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qfont.h>
#include <qcolor.h>
#include <qstring.h>

// KDE includes
#include <kconfig.h>

// Kmud includes
#include "kmudconfigmanager.h"
#include <kmuddebug.h>

// local includes
#include "kmudconfigcache.h"

KmudConfigCache::KmudConfigCache()
{
}

KmudConfigCache::~KmudConfigCache()
{
}

void KmudConfigCache::save()
{
	KConfig* config = KmudConfigManager::self()->configAPI()->kmudConfig();
	writeOptions(config);
}

void KmudConfigCache::load()
{
	KConfig* config = KmudConfigManager::self()->configAPI()->kmudConfig();
	readOptions(config);
}

void KmudConfigCache::readOptions(KConfig *config)
{
	config->setGroup("Fonts");
	outputFont=QFont("fixed");
	outputFont.setPointSize(10);
	outputFont = config->readFontEntry("OutputFont",&outputFont);
	inputFont=QFont("helvetica");
	inputFont.setPointSize(10);
	inputFont = config->readFontEntry("InputFont",&inputFont);

	config->setGroup("Colors");
	defaultBackgroundColor=Qt::black;
	defaultBackgroundColor = config->readColorEntry("DefaultBackground",&defaultBackgroundColor);
	defaultTextColor=QColor(0,255,0);
	defaultTextColor = config->readColorEntry("DefaultText",&defaultTextColor);
    localEchoColor=Qt::red;
	localEchoColor=config->readColorEntry("LocalEchoColor",&localEchoColor);
	debugHighlight=Qt::green;
	debugHighlight=config->readColorEntry("DebugHighLight",&debugHighlight);

	// Background Colors
	backgroundAnsiBlack=Qt::black;
	backgroundAnsiBlack=config->readColorEntry("BackgroundBlack",&backgroundAnsiBlack);
	backgroundAnsiBlue=QColor(0, 0, 128);
	backgroundAnsiBlue=config->readColorEntry("BackgroundBlue",&backgroundAnsiBlue);
	backgroundAnsiRed=QColor(128, 0, 0);
	backgroundAnsiRed=config->readColorEntry("BackgroundRed",&backgroundAnsiRed);
	backgroundAnsiMagenta=QColor(128, 0, 128);
	backgroundAnsiMagenta=config->readColorEntry("BackgroundMagenta",&backgroundAnsiMagenta);
	backgroundAnsiGreen=QColor(0, 128, 0);
	backgroundAnsiGreen=config->readColorEntry("BackgroundGreen",&backgroundAnsiGreen);
	backgroundAnsiCyan=QColor(0, 128, 128);
	backgroundAnsiCyan=config->readColorEntry("BackgroundCyan",&backgroundAnsiCyan);
	backgroundAnsiYellow=QColor(128, 128, 0);
	backgroundAnsiYellow=config->readColorEntry("BackgroundYellow",&backgroundAnsiYellow);
	backgroundAnsiWhite=QColor(192, 192, 192);
	backgroundAnsiWhite=config->readColorEntry("BackgroundWhite",&backgroundAnsiWhite);

	// Foreground Colors
	foregroundAnsiBlack=Qt::black;
	foregroundAnsiBlack=config->readColorEntry("ForegroundBlack",&foregroundAnsiBlack);
	foregroundAnsiGray=QColor(128, 128, 128);
	foregroundAnsiGray=config->readColorEntry("ForegroundGray",&foregroundAnsiGray);
	foregroundAnsiDarkRed=QColor(128, 0, 0);
	foregroundAnsiDarkRed=config->readColorEntry("ForegroundDarkRed",&foregroundAnsiDarkRed);
	foregroundAnsiLightRed=QColor(255, 0, 0);
	foregroundAnsiLightRed=config->readColorEntry("ForegroundLightRed",&foregroundAnsiLightRed);
	foregroundAnsiDarkGreen=QColor(0, 128, 0);
	foregroundAnsiDarkGreen=config->readColorEntry("ForegroundDarkGreen",&foregroundAnsiDarkGreen);
	foregroundAnsiLightGreen=QColor(0, 255, 0);
	foregroundAnsiLightGreen=config->readColorEntry("ForegroundLightGreen",&foregroundAnsiLightGreen);
	foregroundAnsiDarkYellow=QColor(128, 128, 0);
	foregroundAnsiDarkYellow=config->readColorEntry("ForegroundDarkYellow",&foregroundAnsiDarkYellow);
	foregroundAnsiLightYellow=QColor(255, 255, 0);
	foregroundAnsiLightYellow=config->readColorEntry("ForegroundLightYellow",&foregroundAnsiLightYellow);
	foregroundAnsiDarkBlue=QColor(0, 0, 128);
	foregroundAnsiDarkBlue=config->readColorEntry("ForegroundDarkBlue",&foregroundAnsiDarkBlue);
	foregroundAnsiLightBlue=QColor(0, 0, 255);
	foregroundAnsiLightBlue=config->readColorEntry("ForegroundLightBlue",&foregroundAnsiLightBlue);
	foregroundAnsiDarkMagenta=QColor(128, 0, 128);
	foregroundAnsiDarkMagenta=config->readColorEntry("ForegroundDarkMagenta",&foregroundAnsiDarkMagenta);
	foregroundAnsiLightMagenta=QColor(255, 0, 255);
	foregroundAnsiLightMagenta=config->readColorEntry("ForegroundLightMagenta",&foregroundAnsiLightMagenta);
	foregroundAnsiDarkCyan=QColor(0, 128, 128);
	foregroundAnsiDarkCyan=config->readColorEntry("ForegroundDarkCyan",&foregroundAnsiDarkCyan);
	foregroundAnsiLightCyan=QColor(0, 255, 255);
	foregroundAnsiLightCyan=config->readColorEntry("ForegroundLightCyan",&foregroundAnsiLightCyan);
	foregroundAnsiDarkWhite=QColor(192, 192, 192);
	foregroundAnsiDarkWhite=config->readColorEntry("ForegroundDarkWhite",&foregroundAnsiDarkWhite);
	foregroundAnsiLightWhite=QColor(255, 255, 255);
	foregroundAnsiLightWhite=config->readColorEntry("ForegroundLightWhite",&foregroundAnsiLightWhite);

	// Echo Settings
	config->setGroup("Echo");
	localEcho = (EchoState)config->readNumEntry("LocalEcho",(int)Auto);
	inputEcho = (EchoState)config->readNumEntry("InputEcho",(int)Auto);
	suppressNewLine = config->readBoolEntry("SuppressNewLine",false);

	// Timing Settings
	config->setGroup("Timing");
	autoReconnect = config->readBoolEntry("AutoReconnect",true);
	autoReconnectSeconds = config->readNumEntry("AutoReconnectSeconds",10);
	outputNotificationOn = config->readBoolEntry("OutputNotificationOn",true);
	outputNotifyWaitSeconds = config->readNumEntry("OutputNotifySeconds",10);
	connectionTimeoutSeconds = config->readNumEntry("ConnectionTimeoutSeconds",10);

	// General Settings
	config->setGroup("Input");
	keepLastInput = config->readBoolEntry("KeepLastInput",false);
	selectLastInput = config->readBoolEntry("SelectLastInput",false);
	dualInput = config->readBoolEntry("DualInput",false);
	charCmdSeperator = config->readEntry("CharCmdSeperator",";");
	charCmdQuoter = config->readEntry("CharCmdQuoter","#");
	charSpeedwalking = config->readEntry("CharSpeedwalking",".");

}

void KmudConfigCache::writeOptions(KConfig *config)
{
	ConfigAPI* configAPI = KmudConfigManager::self()->configAPI();

	config->setGroup("Fonts");
	config->writeEntry("OutputFont",outputFont);
	config->writeEntry("InputFont",inputFont);

	config->setGroup("Colors");
	config->writeEntry("DefaultBackground",defaultBackgroundColor);
	config->writeEntry("DefaultText",defaultTextColor);
	config->writeEntry("LocalEchoColor",localEchoColor);
	config->writeEntry("DebugHighLight",debugHighlight);

	// Background Colors
	config->writeEntry("BackgroundBlack",backgroundAnsiBlack);
	config->writeEntry("BackgroundBlue",backgroundAnsiBlue);
	config->writeEntry("BackgroundRed",backgroundAnsiRed);
	config->writeEntry("BackgroundMagenta",backgroundAnsiMagenta);
	config->writeEntry("BackgroundGreen",backgroundAnsiGreen);
	config->writeEntry("BackgroundCyan",backgroundAnsiCyan);
	config->writeEntry("BackgroundYellow",backgroundAnsiYellow);
	config->writeEntry("BackgroundWhite",backgroundAnsiWhite);

	// Foreground Colors
	config->writeEntry("ForegroundBlack",foregroundAnsiBlack);
	config->writeEntry("ForegroundGray",foregroundAnsiGray);
	config->writeEntry("ForegroundDarkRed",foregroundAnsiDarkRed);
	config->writeEntry("ForegroundLightRed",foregroundAnsiLightRed);
	config->writeEntry("ForegroundDarkGreen",foregroundAnsiDarkGreen);
	config->writeEntry("ForegroundLightGreen",foregroundAnsiLightGreen);
	config->writeEntry("ForegroundDarkYellow",foregroundAnsiDarkYellow);
	config->writeEntry("ForegroundLightYellow",foregroundAnsiLightYellow);
	config->writeEntry("ForegroundDarkBlue",foregroundAnsiDarkBlue);
	config->writeEntry("ForegroundLightBlue",foregroundAnsiLightBlue);
	config->writeEntry("ForegroundDarkMagenta",foregroundAnsiDarkMagenta);
	config->writeEntry("ForegroundLightMagenta",foregroundAnsiLightMagenta);
	config->writeEntry("ForegroundDarkCyan",foregroundAnsiDarkCyan);
	config->writeEntry("ForegroundLightCyan",foregroundAnsiLightCyan);
	config->writeEntry("ForegroundDarkWhite",foregroundAnsiDarkWhite);
	config->writeEntry("ForegroundLightWhite",foregroundAnsiLightWhite);

	// Echo Settings
	config->setGroup("Echo");
	config->writeEntry("LocalEcho",(int)localEcho);
	config->writeEntry("InputEcho",(int)inputEcho);
	config->writeEntry("SuppressNewLine",suppressNewLine);

	// Timing Settings
	config->setGroup("Timing");
	config->writeEntry("AutoReconnect",autoReconnect);
	config->writeEntry("AutoReconnectSeconds",autoReconnectSeconds);
	config->writeEntry("OutputNotificationOn",outputNotificationOn);
	config->writeEntry("OutputNotifySeconds",outputNotifyWaitSeconds);
	config->writeEntry("ConnectionTimeoutSeconds",connectionTimeoutSeconds);

	// General Settings
	config->setGroup("Input");
	config->writeEntry("KeepLastInput",keepLastInput);
	config->writeEntry("SelectLastInput",selectLastInput);
	config->writeEntry("DualInput",dualInput);	
	config->writeEntry("CharCmdSeperator",charCmdSeperator);
	config->writeEntry("CharCmdQuoter",charCmdQuoter);
	config->writeEntry("CharSpeedwalking",charSpeedwalking);

	KmudConfigManager::self()->blockSignals(true);
	configAPI->emitConfigGroupChanged("Fonts");
	configAPI->emitConfigGroupChanged("Colors");
	configAPI->emitConfigGroupChanged("Echo");
	configAPI->emitConfigGroupChanged("Timing");
	configAPI->emitConfigGroupChanged("General");
	KmudConfigManager::self()->blockSignals(false);
}

