/***************************************************************************
                               kmudconfigmanager.h
                             -------------------
    begin                : Thu Aug 23 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMUDCONFIGMANAGER_H
#define KMUDCONFIGMANAGER_H

// Qt includes
#include <qdict.h>
#include <qstringlist.h>

// Kmud includes
#include <kmudapi.h>

// forward class declarations
class KConfig;
class QDomElement;
class CMudProfile;
class CProfile;
class KmudConfigCache;
class ConfigAPIImpl;

/** This class is a singleton to ensure all shared files
  * are only opened once.
  * @author Kmud Developer Team
  */

class KmudConfigManager : public ProfileAPI {
	Q_OBJECT
public:
	static KmudConfigManager* self();
	~KmudConfigManager();
	
	ConfigAPI* configAPI();

	KmudConfigCache* configCache();

	void setDefaultPath(const QString& defaultPath);
	QString defaultPath() const { return m_defaultPath; }

	// profile API interface
	QDomElement openProfile(const QString& name, bool create = false);
	bool saveProfile(const QString& name);

	QStringList listProfiles(const QString& baseProfile = QString::null) const;

	QStringList listProfileDirs(const QString& mudProfile) const;

	QString defaultSavePath(const QString& mudProfile);
	
private:
	static KmudConfigManager* m_self;
	
	QDict<CMudProfile> m_mudProfiles;
	
	QString m_defaultPath;   // path profiles get save to
	QStringList m_paths;	 // list of paths where profiles will be searched

	ConfigAPIImpl* m_configAPI;

private:
	// constructor is private in singletons
	KmudConfigManager();

	/**
	  * Load the Mud profile for the MUD mudName.
	  * Returns 0 if the profile cannot be loaded.
	  */
	CMudProfile* loadMudProfile(const QString& mudName);

	/**
	  * Load the Character profile for the MUD mudName and teh Character charName.
	  * Returns 0 if the profile cannot be loaded.
	  */
	CProfile* loadCharProfile(const QString& mudName, const QString& charName);

	/**
	  * Creates the relative filename of a profile, encoding names when necessary.
	  * If charName is empty the relative filename will be mudname/mudname.kmudprf
	  * If charName is not empty the relative filename will be mudname/characters/charnam.kmudprf
	  */
	QString profileFilename(const QString& mudName, const QString& charName = QString::null) const;

	/**
	  * Searches through the list of possible paths to find a profile.
	  * It returns a list of filenames which match the search parameters.
	  */
	QStringList findProfile(const QString& mudName, const QString& charName = QString::null) const;

	/**
	  * Creates the save filename for a given profile.
	  * All necessary directories are created.
	  */
	QString profileSaveFilename(const QString& mudName, const QString& charName = QString::null);

	/**
	  * Lists all the profile files (*.kmudprf) in a given directory.
	  */
	QStringList listProfileFiles(const QString& directory) const;

private slots:
	/**
	  * Updates the KmudConfigCache instance if a cached group was changed.
	  */
	void slotConfigDataChanged(const QString& group);
};

#endif
