/***************************************************************************
                               kmudconnecthistory.cpp
                             -------------------
    begin                : Thu Jun 6 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmudconnecthistory.h"

#include <klocale.h>

#include "kmudmainwindow.h"
#include "ctelnet.h"

#include <kmuddebug.h>

QStringList KmudConnectHistory::m_lstConnections;
uint KmudConnectHistory::m_maxItems = 5;
QPtrList<KmudConnectHistory> KmudConnectHistory::m_instanceList;

KmudConnectHistory::KmudConnectHistory(KmudMainWindow *window,QObject *parent) : KListAction(i18n("Recent Connections"),0,parent,"mudConnectHistory")
{
	m_instanceList.setAutoDelete(false);
	m_instanceList.append(this);	

	setItems(m_lstConnections);
	connect( this, SIGNAL( activated( const QString& ) ),
           window, SLOT( slotRecentConnectionSelected( const QString& ) ) );
}

KmudConnectHistory::~KmudConnectHistory()
{
	m_instanceList.remove(this);
}

/** Get the list of connections in the connection hisotry
  * @return The connection list */
QStringList *KmudConnectHistory::getConnections(void)
{
	return &m_lstConnections;
}

/** Used to add a conenction to the history
  * @param connection The connetion to add to the history  */
void KmudConnectHistory::addConnection(QString connection)
{
	// QStringList lst = items();

    m_lstConnections.remove( connection );

    // remove las item if already maxitems in list
    if( m_lstConnections.count() == m_maxItems )
    {
        // remove last item
        m_lstConnections.remove( m_lstConnections.last() );
    }

    m_lstConnections.prepend( connection );

	for (KmudConnectHistory *i = m_instanceList.first(); i!=NULL; i = m_instanceList.next())
	{
		i->setItems(m_lstConnections);
	}
}

void KmudConnectHistory::saveEntries( KConfig* config)
{
	QString     key;
	QString     value;
	QString     oldGroup;
	QString groupname = "RecentConnections";

	oldGroup = config->group();

	config->setGroup( groupname );

	// write connections list
    for( unsigned int i = 1 ; i <= m_lstConnections.count() ; i++ )
    {
        key = QString( "Connection%1" ).arg( i );
        value = m_lstConnections[ i - 1 ];
        config->writeEntry( key, value );
    }

    config->setGroup( oldGroup );
}

void KmudConnectHistory::loadEntries( KConfig* config)
{
	if (m_lstConnections.count()==0)
	{
		QString     key;
		QString     value;
		QString     oldGroup;
		QString groupname = "RecentConnections";

		oldGroup = config->group();

		
		config->setGroup( groupname );

		// read file list
		for( unsigned int i = 1 ; i <= m_maxItems ; i++ )
		{
			key = QString( "Connection%1" ).arg( i );
			value = config->readEntry( key, QString::null );

			if (!value.isNull())
			{
				m_lstConnections.append( value );
			}
		}

		// set connections
		for (KmudConnectHistory *i = m_instanceList.first(); i!=NULL; i = m_instanceList.next())
		{
			i->setItems(m_lstConnections);
		}	

		config->setGroup( oldGroup );
	}
}
