/***************************************************************************
                               kmudview.cpp
                             -------------------
    begin                : Sat Mar  3 19:17:48 CET 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qpainter.h>
#include <qlineedit.h>

// include files for KDE
#include <klocale.h>

// Kmud specific includes
#include "kmudview.h"
#include "kmudmainwindow.h"
#include "kmuddebug.h"
#include "cmudview.h"
#include "kmudviewfilter.h"


KmudView::KmudView(QWidget *parent, const char *name) : QWidget(parent, name)
{
	setBackgroundMode(PaletteBase);

	m_layout = new QVBoxLayout(this);
	m_outputView = new CMudView(this, "OutputView");
	m_layout->addWidget(m_outputView);

	m_dualInput = false;
	m_input1 = new KHistoryCombo(true, this,"Input1");
	m_input2 = new KHistoryCombo(true, this,"Input2");
	m_input2->hide();
	m_layout->addWidget(m_input1);
	m_layout->addWidget(m_input2);
	m_localEchoMode = KmudConfigCache::Auto;
	m_localEchoColor = red;
	m_localEcho = true;
	m_inputEcho = true;
	m_inputEchoMode = KmudConfigCache::Auto;
	m_suppressNewLine = false;
	m_keepLastInput = false;
	m_selectLastInput = true;

	m_input1->setFocus();

	connect(m_input1, SIGNAL(returnPressed()), this, SLOT(slotInputReturnPressed1()) );
	connect(m_input2, SIGNAL(returnPressed()), this, SLOT(slotInputReturnPressed2()) );
	connect(m_input1, SIGNAL(activated(const QString&)), this, SLOT(slotInputActivated1(const QString&)));
	connect(m_input2, SIGNAL(activated(const QString&)), this, SLOT(slotInputActivated2(const QString&)));

	m_filter = new KmudViewFilter(this);
}

KmudView::~KmudView()
{
	delete m_filter;
	delete m_input1;
	delete m_input2;
	delete m_outputView;
	delete m_layout;
}

void KmudView::slotInputActivated1(const QString& s)
{
	m_input1->addToHistory(s);
	if (m_selectLastInput && m_keepLastInput)
	{
		m_input1->lineEdit()->selectAll();
	}
}

void KmudView::slotInputActivated2(const QString& s)
{
	m_input2->addToHistory(s);
	if (m_selectLastInput && m_keepLastInput)
	{
		m_input2->lineEdit()->selectAll();
	}
}

/** This method is used to set the input echo mode */
void KmudView::setInputEchoMode(KmudConfigCache::EchoState mode)
{
	m_inputEchoMode = mode;

	if ((m_inputEchoMode==KmudConfigCache::Off) || ((m_inputEchoMode==KmudConfigCache::Auto) && (m_inputEcho==false)))
	{
		m_input1->lineEdit()->setEchoMode(QLineEdit::NoEcho);
		m_input2->lineEdit()->setEchoMode(QLineEdit::NoEcho);
	}
	else
	{
		m_input1->lineEdit()->setEchoMode(QLineEdit::Normal);
		m_input2->lineEdit()->setEchoMode(QLineEdit::Normal);
	}
}

/** This is used to set if the input line should display text entered into it */
void KmudView::setInputEcho(bool echo)
{
	m_inputEcho = echo;

	if ((m_inputEchoMode==KmudConfigCache::Off) || ((m_inputEchoMode==KmudConfigCache::Auto) && (m_inputEcho==false)))
	{
		m_input1->lineEdit()->setEchoMode(QLineEdit::NoEcho);
		m_input2->lineEdit()->setEchoMode(QLineEdit::NoEcho);
	}
	else
	{
		m_input1->lineEdit()->setEchoMode(QLineEdit::Normal);
		m_input2->lineEdit()->setEchoMode(QLineEdit::Normal);
	}

}

/** This method is used to set the local echo mode */
void KmudView::setLocalEchoMode(KmudConfigCache::EchoState mode)
{
	m_localEchoMode = mode;
}

/** This method is when echo is turned on or off by the CTelnet class which will
	  * recived the commands from the mud
	  */
void KmudView::setLocalEcho(bool echo)
{
	m_localEcho = echo;
}

/** This method is used to set the color of the local echo */
void KmudView::setLocalEchoColor(QColor color)
{
	m_localEchoColor = color;
}

/**
  * This method is used to set the input line font
  */
void KmudView::setInputFont(QFont font)
{
	m_input1->setFont(font);
	m_input2->setFont(font);
}


/**
 * This is used to add text to the mud view in the local echo color
 * @param text The text to add
 */
void KmudView::addEchoText(QString text)
{
	QColor textColor = m_outputView->textColor();
	m_outputView->setTextColor(m_localEchoColor);
	m_outputView->addText(text);
	m_outputView->setTextColor(textColor);
	
}

void KmudView::addText(QString text)
{
	m_outputView->addText(text);
}

KmudFilter* KmudView::filter() const
{
	return m_filter;
}

void KmudView::slotInputReturnPressed1()
{
	slotInputReturnPressed(m_input1);
}

void KmudView::slotInputReturnPressed2()
{
	slotInputReturnPressed(m_input2);
}

void KmudView::slotInputReturnPressed(KHistoryCombo* inputLine)
{
	QString input = inputLine->currentText();
	if ((m_localEchoMode==KmudConfigCache::On) || ((m_localEcho == true) && (m_localEchoMode==KmudConfigCache::Auto)))
	{
		input = input +"\n";

		addEchoText(input);
	}
	else
	{
		if (!m_suppressNewLine)
		{
			input = input +"\n";
		}
	}

	emit userInput(input);

	if (!((m_inputEchoMode==KmudConfigCache::On) || ((m_inputEcho == true) && (m_inputEchoMode==KmudConfigCache::Auto))))
	{
		inputLine->clearEdit();
	}

	if (!m_keepLastInput)	
	{
		inputLine->clearEdit();
	}
}

/** This method is used to set if the input should be kept when return is pressed */
void KmudView::setKeepLastInput(bool keep)
{
	m_keepLastInput = keep;
}

/** This method is used to set if the input should be selected after pressig return */
void KmudView::setSelectLastInput(bool select)
{
	m_selectLastInput = select;
}
 
/** This is used to suppress new lines when echo is off */
void KmudView::setSuppressNewLine(bool suppress)
{
	m_suppressNewLine = suppress;
}

/** This method is used to set if dual input lines are to be used */
void KmudView::setDualInputLines(bool dual)
{
	m_dualInput = dual;

	if (m_dualInput)
		m_input2->show();
	else
		m_input2->hide();
}

/** Set the defaults for Font, text color anf background color.  */
void KmudView::init(const QFont font, const QColor textColor, const QColor bgColor)
{
	m_outputView->init(font, textColor, bgColor);
}

/** Clears the view and resets font and color settings to the default values.
If no default values were set, the font defaults to the QFont default, text color to red
and background color to black. */
void KmudView::clear()
{
	m_outputView->clear();
	m_filter->retrieveDefaults();
}

/** Shows the Kmud splash document, i.e. app_dir/kmud/splash/splash.html */
void KmudView::splash()
{
	// m_outputView->splash();
}


/** Sets the font for all following addText operations until the next setFont() or clear() call.
Status of the style setting (bold, underline, italic) of the QFont object will be ignored. */
void KmudView::setFont(QFont font, bool useStyle)
{
	m_outputView->setFont(font, useStyle);
}

/** Returns the currently used font, including the style settings.  */
QFont KmudView::font()
{
	return m_outputView->font();
}

/** Sets the underline style on if underline is true */
void KmudView::setUnderline(bool underline)
{
	m_outputView->setUnderline(underline);
}
	
/** Returns true if underline is active */
bool KmudView::underline()
{
	return m_outputView->underline();
}

/** Sets the italic style on if italic is true */
void KmudView::setItalic(bool italic)
{
	m_outputView->setItalic(italic);
}

/** Returns true if italic is active */
bool KmudView::italic()
{
	return m_outputView->italic();
}

/** Sets the bold style on if bold is true */
void KmudView::setBold(bool bold)
{
	m_outputView->setBold(bold);
}

/** Returns true if bold is active */
bool KmudView::bold()
{
	return m_outputView->bold();
}

/** Sets the text color. */
void KmudView::setTextColor(QColor color)
{
	m_outputView->setTextColor(color);
}

/** Returns the currently used text color. */
QColor KmudView::textColor()
{
	return m_outputView->textColor();
}

/** Sets the text background color. */
void KmudView::setBgColor(QColor color)
{
	m_outputView->setBgColor(color);
}

/** Returns the currently used text background color. */
QColor KmudView::bgColor()
{
	return m_outputView->bgColor();
}

void KmudView::setBlink(bool state)
{
	m_outputView->setBlink(state);
}

bool KmudView::blink()
{
	return m_outputView->blink();
}

/**
  * If enable is true, the view will update everytime addText is called.
  * This might be undesired when addText is called more than once per line.
  */
void KmudView::setUpdatesEnabled(bool enable)
{
	// m_outputView->setUpdatesEnabled(enable);
}

/**
  * Returns true when the view's content will be updated on every addText(const QString) call.
  * This can be set by the setUpdatesEnabled(bool) method.
  */
bool KmudView::updatesEnabled()
{
	// return m_outputView->updatesEnabled();
	return true;
}
