/***************************************************************************
                               kmudviewfilter.h
                             -------------------
    begin                : Tue Apr 30 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMUDVIEWFILTER_H
#define KMUDVIEWFILTER_H

// Qt includes
#include <qcolor.h>

// Kmud includes
#include <kmudfilter.h>
#include "cansiinterpreter.h"

// forward declarations
class CANSIParser;
class KmudView;
class KmudConfigCache;

/**The filter that removes the ANSI sequences from the
  *mud output and sets the views properties accordingly.
  *@author Kmud Developer Team
  */

class KmudViewFilter : public KmudFilter, public CANSIInterpreter {
public: 
	KmudViewFilter(KmudView* view);
	~KmudViewFilter();

	QString filterOutput(QString& input);

	// ANSI call backs
	void inputText(const QString& text);
	void inputTextColor(ANSI::Color color);
	void inputBackColor(ANSI::Color color);
	void inputTextAttribute(ANSI::TextAttr attribute);

	/** Use the view's current setting as the default state for later operations. */
	void retrieveDefaults();

private:
	CANSIParser* m_parser;
	KmudView* m_view;
	KmudConfigCache* cache;

	QString m_buffer;
	QColor m_defaultTextColor;
	QColor m_defaultBGColor;
	bool m_lightColorsOn;
};

#endif
