/***************************************************************************
                               cmudstream.cpp
                             -------------------
    begin                : Fri Aug 10 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmudstream.h"

#include <kmuddebug.h>
#include <kmudfilter.h>


CMudStream::CMudStream(QObject* parent, const char* name)
:QObject(parent,name)
{
	m_inputFilterList.setAutoDelete(false);
	m_outputFilterList.setAutoDelete(false);
}

CMudStream::~CMudStream()
{
}


int CMudStream::calculateFilterPosition(QPtrList<KmudFilter> filterList, const QStringList after, const QStringList before)
{
	int ai,bi,i;
	KmudFilter* f;
	QString s;
	QStringList::ConstIterator it;
	QStringList sl;


	kdDebug(DAREA_LIB_MUDSTREAM) << " calculating filter position..."  << endl;
	s = " after=";
	for (it=after.begin(); it!=after.end(); ++it)
		s = s + (*it).latin1() + " ";
	kdDebug(DAREA_LIB_MUDSTREAM) << s << endl;
	s = " before=";
	for (it=before.begin(); it!=before.end(); ++it)
		s = s + (*it).latin1() + " ";
	kdDebug(DAREA_LIB_MUDSTREAM) << s << endl;


	ai=0;
	bi=filterList.count();

	// look for every filter name if it contains one of the strings in the stringlists
	// it's not an exact match (intentionlly)
	for (f=filterList.first(); f!=0; f=filterList.next())
	{
		for (it=after.begin(); it!=after.end(); ++it)
			if (f->filterName().contains((*it),false))
				ai = filterList.at() + 1;

		for (it=before.begin(); it!=before.end(); ++it)
			if (f->filterName().contains((*it),false))
				bi = filterList.at();
	}

	// handle keyword "all" (or "All", "ALL", ...)
	for (it=after.begin(); it!=after.end(); ++it)
		if ((*it).lower() == "all")
			ai = filterList.count();
	for (it=before.begin(); it!=before.end(); ++it)
		if ((*it).lower() == "all")
			bi = 0;

	if (ai > bi)
	{
		kdDebug(DAREA_LIB_MUDSTREAM) << " requirements couldn't be met. ai=" << ai << " bi=" << bi << endl;
		i = -1;
	}
	else
	{
		i = ai;
	}

	kdDebug(DAREA_LIB_MUDSTREAM) << " calculated position: " << i << endl;

	return i;
}

bool CMudStream::registerInputFilter(const KmudFilter* f, const QStringList after, const QStringList before)
{
	int pos;
	bool retVal;

	kdDebug(DAREA_LIB_MUDSTREAM) << "registerInputFilter (" << f->filterName() << ")" << endl;
	pos = calculateFilterPosition(m_inputFilterList,after,before);
	if (pos >= 0)
		retVal = m_inputFilterList.insert(pos, f);
	else
		retVal = false;

	if (retVal == false)
		kdDebug(DAREA_LIB_MUDSTREAM) << "ERROR" << endl;

	return retVal;
}

bool CMudStream::registerOutputFilter(const KmudFilter* f, const QStringList after, const QStringList before)
{
	int pos;
	bool retVal;

	kdDebug(DAREA_LIB_MUDSTREAM) << "registerOutputFilter (" << f->filterName() << ")" << endl;
	pos = calculateFilterPosition(m_outputFilterList,after,before);
	if (pos >= 0)
		retVal = m_outputFilterList.insert(pos, f);
	else
		retVal = false;

	if (retVal == false)
		kdDebug(DAREA_LIB_MUDSTREAM) << "ERROR" << endl;

	return retVal;
}

bool CMudStream::removeInputFilter(const KmudFilter* f)
{
	bool retVal;
	unsigned int i, num;

	kdDebug(DAREA_LIB_MUDSTREAM) << "removeInputFilter (" << f->filterName() << ")" << endl;

	retVal = true;
	num = m_inputFilterList.contains(f);
	i = 1;
	while ((i<=num) && (retVal==true))
	{
		retVal = m_inputFilterList.remove(f);
		i++;
	}
	if (retVal == false)
		kdDebug(DAREA_LIB_MUDSTREAM) << "ERROR" << endl;

	return retVal;
}

bool CMudStream::removeOutputFilter(const KmudFilter* f)
{
	bool retVal;
	unsigned int i, num;

	kdDebug(DAREA_LIB_MUDSTREAM) << "removeOutputFilter (" << f->filterName() << ")" << endl;

	retVal = true;
	num = m_outputFilterList.contains(f);
	i = 1;
	while ((i<=num) && (retVal==true))
	{
		retVal = m_outputFilterList.remove(f);
		i++;
	}
	if (retVal == false)
		kdDebug(DAREA_LIB_MUDSTREAM) << "ERROR" << endl;

	return retVal;
}

void CMudStream::processInput(QString _input)
{
	KmudFilter* f;
	QString s_in, s_out;

	s_in = _input;

	for ( f=m_inputFilterList.first(); f!=0; f=m_inputFilterList.next() )
	{
		kdDebug(DAREA_LIB_MUDSTREAM) << "filterInput (" << f->filterName() << ")" << endl;
		s_out = f->filterInput(s_in);
		s_in = s_out;
	}

	emit(inputProcessed());
}

void CMudStream::processOutput(QString _output)
{
	KmudFilter* f;
	QString s_in, s_out;

	s_in = _output;

	for ( f=m_outputFilterList.first(); f!=0; f=m_outputFilterList.next() )
	{
		kdDebug(DAREA_LIB_MUDSTREAM) << "filterOutput (" << f->filterName() << ")" << endl;
		s_out = f->filterOutput(s_in);
		s_in = s_out;
	}

	emit(outputProcessed());
}



QStringList CMudStream::inputFiltersBefore(const KmudFilter* f)
{
	QStringList sl;
	KmudFilter* f2;
	bool done;

	done = false;
	f2=m_inputFilterList.first();

	while ((done==false) || (f2!=0))
	{
		if (f2->filterName() == f->filterName())
			done = true;
		else sl.append(f2->filterName());

		f2 = m_inputFilterList.next();
	}

	return sl;
}

QStringList CMudStream::inputFiltersAfter(const KmudFilter* f)
{
	QStringList sl;
	KmudFilter* f2;
	bool done;

	done = false;
	f2=m_inputFilterList.last();

	while ((done==false) || (f2!=0))
	{
		if (f2->filterName() == f->filterName())
			done = true;
		else sl.append(f2->filterName());

		f2 = m_inputFilterList.prev();
	}

	return sl;
}

QStringList CMudStream::outputFiltersBefore(const KmudFilter* f)
{
	QStringList sl;
	KmudFilter* f2;
	bool done;

	done = false;
	f2=m_outputFilterList.first();

	while ((done==false) || (f2!=0))
	{
		if (f2->filterName() == f->filterName())
			done = true;
		else sl.append(f2->filterName());

		f2 = m_outputFilterList.next();
	}

	return sl;
}

QStringList CMudStream::outputFiltersAfter(const KmudFilter* f)
{
	QStringList sl;
	KmudFilter* f2;
	bool done;

	done = false;
	f2=m_outputFilterList.last();

	while ((done==false) || (f2!=0))
	{
		if (f2->filterName() == f->filterName())
			done = true;
		else sl.append(f2->filterName());

		f2 = m_outputFilterList.prev();
	}

	return sl;
}

