/***************************************************************************
                               kmemconfig.cpp
                             -------------------
    begin                : Thu Jun 13 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmemconfig.h"

#include <kmuddebug.h>

void KMemConfig::printDebugOutput(QString group)
{
	typedef QMap<QString, QString> EntryMap;
	kdDebug(DAREA_LIB) << "===================================================" << endl;
	kdDebug(DAREA_LIB) << "  Entries for KMemConfig group : " << group << endl;
	kdDebug(DAREA_LIB) << "===================================================" << endl;
    EntryMap entries = entryMap(group);
	for (EntryMap::ConstIterator it = entries.begin(); it != entries.end(); ++it)
	{
		kdDebug(DAREA_LIB) << "  " << it.key() <<"='"<<it.data() << "'" << endl;
	}
	kdDebug(DAREA_LIB) << "===================================================" << endl;
}

void KMemConfig::copyTo(QString fromGroup,KMemConfig *newProp)
{
	typedef QMap<QString, QString> EntryMap;

	EntryMap entries = entryMap(fromGroup);
	for (EntryMap::ConstIterator it = entries.begin(); it != entries.end(); ++it)
	{
		newProp->writeEntry(it.key(),it.data());
	}
}

KMemConfig::KMemConfig() : KConfigBase()
{
	setReadOnly(false);
}

KMemConfig::~KMemConfig()
{
}

QStringList KMemConfig::groupList() const
{
	QStringList retList;

	KEntryMapConstIterator aIt;
	KEntryMapConstIterator aEnd = aEntryMap.end();
	for (aIt = aEntryMap.begin(); aIt != aEnd; ++aIt)
	{
		while(aIt.key().mKey.isEmpty())
		{
			QCString group = aIt.key().mGroup;
			++aIt;
			while (true)
			{
				if (aIt == aEnd)
					return retList; // done

				if (aIt.key().mKey.isEmpty())
					break; // Group is empty, next group

				if (!aIt.key().bDefault && !(*aIt).bDeleted)
				{
					retList.append(QString::fromUtf8(group));
					break; // Grou is non-empty, added, next gropup
				}
				++aIt;
			}
		}
	}

	return retList;
}

QMap<QString, QString> KMemConfig::entryMap(const QString &pGroup) const
{
	QCString pGroup_utf = pGroup.utf8();
	QMap<QString, QString> tmpMap;
	KEntryMapConstIterator aIt;
	KEntry aEntry;
	KEntryKey groupKey( pGroup_utf, 0 );

	aIt = aEntryMap.find(groupKey);
	if (aIt == aEntryMap.end())
		return tmpMap;
	++aIt; // advance past special group entry marker
	for (; aIt.key().mGroup == pGroup_utf && aIt != aEntryMap.end(); ++aIt)
	{
		// Leave the default values out && leave deleted entries out
		if (!aIt.key().bDefault && !(*aIt).bDeleted)
			tmpMap.insert(QString::fromUtf8(aIt.key().mKey), QString::fromUtf8((*aIt).mValue.data(), (*aIt).mValue.length()));
	}

	return tmpMap;
}

void KMemConfig::reparseConfiguration()
{
	// Don't do anything
}

KEntryMap KMemConfig::internalEntryMap(const QString &pGroup) const
{
	QCString pGroup_utf = pGroup.utf8();
	KEntry aEntry;
	KEntryMapConstIterator aIt;
	KEntryKey aKey(pGroup_utf, 0);
	KEntryMap tmpEntryMap;

	aIt = aEntryMap.find(aKey);
	if (aIt == aEntryMap.end())
	{
		// the special group key is not in the map,
		// so it must be an invalid group.  Return
		// an empty map.
		return tmpEntryMap;
	}

	// we now have a pointer to the nodes we want to copy.
	for (; aIt.key().mGroup == pGroup_utf && aIt != aEntryMap.end(); ++aIt)
	{
		tmpEntryMap.insert(aIt.key(), *aIt);
	}

	return tmpEntryMap;
}

void KMemConfig::putData(const KEntryKey &_key, const KEntry &_data, bool _checkGroup)
{
	// check to see if the special group key is present,
	// and if not, put it in.
	if (_checkGroup)
	{
		KEntryKey groupKey( _key.mGroup, 0);
	    KEntry &entry = aEntryMap[groupKey];
	}

	// now either add or replace the data
	KEntry &entry = aEntryMap[_key];
	entry = _data;

	if (_key.bDefault)
	{
		// We have added the data as default value,
		// add it as normal value as well.
		KEntryKey key(_key);
		key.bDefault = false;
		aEntryMap[key] = _data;
	}

}

KEntry KMemConfig::lookupData(const KEntryKey &_key) const
{
	KEntryMapConstIterator aIt;

	aIt = aEntryMap.find(_key);
	if (aIt != aEntryMap.end())
	{
		const KEntry &entry = *aIt;
		if (entry.bDeleted)
			return KEntry();
		else
		return entry;
	}
	else
	{
		return KEntry();
	}
}

bool KMemConfig::internalHasGroup(const QCString &group) const
{
	KEntryKey groupKey( group, 0);

	KEntryMapConstIterator aIt = aEntryMap.find(groupKey);
	KEntryMapConstIterator aEnd = aEntryMap.end();

	if (aIt == aEnd)
		return false;

	++aIt;
	for(; (aIt != aEnd); ++aIt)
	{
		if (aIt.key().mKey.isEmpty())
			break;

		if (!aIt.key().bDefault && !(*aIt).bDeleted)
			return true;
	}
	return false;
}

