/***************************************************************************
                          dlgmaptextproperties.cpp  -  description
                             -------------------
    begin                : Thu Mar 8 2001
    copyright            : (C) 2001 by KMud Development Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgmaptextproperties.h"

#include <qfile.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qtextstream.h>
#include <qpalette.h>
#include <qtabwidget.h>
#include <qlayout.h>
#include <qfontmetrics.h>

#include <kcolorbutton.h>
#include <knuminput.h>
#include <klocale.h>

#include "../cmaptext.h"
#include "../cmapmanager.h"
#include "../cmapcmdelementcreate.h"
#include "../cmapcmdelementdelete.h"
#include "../cmapcmdelementproperties.h"
#include "../cmapcmdgroup.h"

#include "../cmappluginbase.h"
#include "../cmappropertiespanebase.h"
#include "../cmapviewbase.h"

#include <kmuddebug.h>

#include <X11/Xlib.h>

CMapTextPreview::CMapTextPreview(CMapManager *manager,QWidget *parent, const char *name)
	: QScrollView(parent,name, Qt::WNorthWestGravity | Qt::WResizeNoErase | Qt::WRepaintNoErase)
{
	buffer = NULL;
	mapManager = manager;

	setHScrollBarMode(Auto);
	setVScrollBarMode(Auto);
}

CMapTextPreview::~CMapTextPreview()
{
	if (buffer)
		delete buffer;
}

void CMapTextPreview::drawContents(QPainter *paint,int , int , int, int )
{
	int width,height;

	if (contentsWidth()>viewport()->width())
		width = contentsWidth();
	else
		width = viewport()->width();

	if (contentsHeight()>viewport()->height())
		height = contentsHeight();
	else
		height = viewport()->height();

	QRect drawArea(0,0,width,height);
	
	// delete the buffer only when we need one with a different size
	if (buffer && (buffer->size() != drawArea.size()))
	{
		delete buffer;
		buffer = NULL;
	}

	if (!buffer)
	{
		buffer = new QPixmap(drawArea.size());
	}

	QPainter p;

	p.begin(buffer);

	if (mapManager->getActiveView()->getCurrentlyViewedZone()->getUseDefaultBackground())
	{
		p.fillRect(drawArea,mapManager->getMapData()->backgroundColor);
	}
	else
	{
		p.fillRect(drawArea,mapManager->getActiveView()->getCurrentlyViewedZone()->getBackgroundColor());
	}

	QStrList textList;
	CMapText::stringToList(text,&textList);
	CMapText::paintText(&p,color,QPoint(0,0),font,&textList,size);

	paint->drawPixmap(0,0,*buffer);
}


DlgMapTextProperties::DlgMapTextProperties(CMapManager *manager,CMapText *textElement,QWidget *parent, const char *name ) : DlgMapTextPropertiesBase(parent,name,true)
{
	text = textElement;
	mapManager = manager;
	QString width;
	QString height;
	QVBoxLayout *vbox = new QVBoxLayout((QWidget *)fraPreview);
	textScrollView = new CMapTextPreview(mapManager,fraPreview,"textPreview");
	vbox->addWidget( textScrollView);
	textScrollView->show();
	fillFamilyList();
	setFont(text->getFont());
	width.sprintf("%d",text->getWidth());
	height.sprintf("%d",text->getHeight());
	txtText->setText(text->getText());
	txtWidth->setText(width);
	txtHeight->setText(height);
	cmdColor->setColor(text->getColor());

	//FIXME_jp: set txtText background to background color of the map

	// Get the extension panels from the plugins
	for (CMapPluginBase *plugin=mapManager->getPluginList()->first();
	     plugin!=0;
	     plugin=mapManager->getPluginList()->next())
	{
		QPtrList<CMapPropertiesPaneBase> paneList = plugin->getPropertyPanes(TEXT,(CMapElement*)textElement,(QWidget *)TextTabs);
		for (CMapPropertiesPaneBase *pane = paneList.first();pane!=0;pane = paneList.next())
		{
			TextTabs->addTab(pane,pane->getTitle());
			connect(cmdOk,SIGNAL(clicked()),pane,SLOT(slotOk()));
			connect(cmdCancel,SIGNAL(clicked()),pane,SLOT(slotCancel()));
		}
	}

	slotUpdatePreview();
}

DlgMapTextProperties::~DlgMapTextProperties()
{
}

void DlgMapTextProperties::fillFamilyList(void)
{
	int numFonts;
	Display *kde_display;

	char** fontNames;
	char** fontNames_copy;
	QString qfontname;

	QStrList fontlist(TRUE);

	kde_display = XOpenDisplay( 0L );

	fontNames = XListFonts(kde_display, "*", 32767, &numFonts);
	fontNames_copy = fontNames;
	for(int i = 0; i < numFonts; i++)
	{

	    if (**fontNames != '-')
		{

			// The font name doesn't start with a dash -- an alias
			// so we ignore it. It is debatable whether this is the right
			// behaviour so I leave the following snippet of code around.
			// Just uncomment it if you want those aliases to be inserted as well.

			fontNames ++;
			continue;
		}

		qfontname = "";
	    qfontname = *fontNames;
	    int dash = qfontname.find ('-', 1, TRUE); // find next dash

	    if (dash == -1)
		{
			// No such next dash -- this shouldn't happen.
            // but what do I care -- lets skip it.
			fontNames ++;
			continue;
		}

		// the font name is between the second and third dash so:
		// let's find the third dash:

		int dash_two = qfontname.find ('-', dash + 1 , TRUE);

		if (dash == -1)
		{
			// No such next dash -- this shouldn't happen.
			// But what do I care -- lets skip it.
			fontNames ++;
			continue;
		}

		// fish the name of the font info string
		qfontname = qfontname.mid(dash +1, dash_two - dash -1);

		if( !qfontname.contains("open look", TRUE))
		{
			if(qfontname != "nil")
			{
				if(fontlist.find(qfontname) == -1)
					fontlist.inSort(qfontname);
			}
		}

		fontNames ++;

	}
	
	for(fontlist.first(); fontlist.current(); fontlist.next())
	{	
		QString s;
		s = fontlist.current();
		lstFamily->insertItem(s,-1);
		if (s.stripWhiteSpace() =="times new roman" || s.stripWhiteSpace() =="times")
		{
			lstFamily->setCurrentItem(lstFamily->count()-1);
		}
	}
	
	XFreeFontNames(fontNames_copy);
	XCloseDisplay(kde_display);
}

void DlgMapTextProperties::setFont(QFont font)
{
	unsigned int i;

	textFont = font;
	
	QString family = font.family();
	family = family.stripWhiteSpace();
	QString size;
	size.sprintf("%d",font.pointSize());	
	
	for (i = 0 ; i<lstFamily->count();i++)
	{
		QString s = lstFamily->text(i);
		if (s == family)
		{
			lstFamily->setCurrentItem(i);
			break;
		}
	}	

	chkBold->setChecked(font.bold());
	chkItalic->setChecked(font.italic());
	lstFamily->centerCurrentItem();	
	slotUpdatePreview();
}

void DlgMapTextProperties::slotSetSize(void)
{
	kdDebug(DAREA_PARTS_MAPPER) << "CMapTextPreview::slotSetSize1 " << endl;
	int fontSize = txtFontSize->text().toInt();
	textFont.setPointSize(fontSize);
	QFontMetrics fm(textFont);	
	QString width;
	QString height;
	QStrList textList;
	CMapText::stringToList(txtText->text(),&textList);
	int tmpWidth = 0;
	for (char *s = textList.first(); s!=0; s = textList.next())
	{
		if (fm.width(s) > tmpWidth)
			tmpWidth = fm.width(s);
    }

	width.sprintf("%d",tmpWidth);
	height.sprintf("%d",fm.height() * textList.count());
	txtWidth->setText(width);
	txtHeight->setText(height);
	txtFontSize->setText("");
	slotUpdatePreview();
}

void DlgMapTextProperties::slotBoldClicked(void)
{
	textFont.setBold(chkBold->isChecked());
	slotUpdatePreview();
}

void DlgMapTextProperties::slotItalicClicked(void)
{
	textFont.setItalic(chkItalic->isChecked());
	slotUpdatePreview();
}

void DlgMapTextProperties::slotFamilySelected(int index)						
{
	textFont.setFamily(lstFamily->text(index));
	slotUpdatePreview();
}

void DlgMapTextProperties::slotColorChanged(const QColor &color)
{
	textColor = color;
	slotUpdatePreview();
}

void DlgMapTextProperties::slotAccept()
{
	CMapCmdElementProperties *command = new CMapCmdElementProperties(mapManager,i18n("Changed Room Properties"),text);

	QStrList textList;
	int width = txtWidth->text().toInt();
	int height = txtHeight->text().toInt();

	command->compare("Text",text->getText(),txtText->text());
	command->compare("Color",text->getColor(),textColor);
	command->compare("Font",text->getFont(),textFont);
	command->compare("Size",text->getSize(),QSize(width,height));

	mapManager->addCommand(command);

	accept();
}

void DlgMapTextProperties::slotUpdatePreview()
{
	int gridWidth = mapManager->getMapData()->gridSize.width();
	int gridHeight = mapManager->getMapData()->gridSize.height();
	int width =txtWidth->text().toInt();
	if (width<gridWidth)
		width = gridWidth;
	int height = txtHeight->text().toInt();
	if (height < gridHeight)
		height = 20;

	textScrollView->setFont(textFont);
	textScrollView->setColor(textColor);
	textScrollView->setSize(QSize(width,height));
	textScrollView->setText(txtText->text());
	textScrollView->resizeContents(txtWidth->text().toInt(),txtHeight->text().toInt());
	textScrollView->viewport()->repaint(false);
}
