/***************************************************************************
                               cmapviewmovetracker.h
                             -------------------
    begin                : Fri May 11 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CMAPVIEWMOVETRACKER_H
#define CMAPVIEWMOVETRACKER_H

#include "../../cmapviewbase.h"

class CMapManager;

class QWidget;
class QPaintEvent;
class QPixmap;
class QSize;

/**
  *@author Kmud Developer Team
  */

class CMapViewMoveTracker : public CMapViewBase
{
   Q_OBJECT
public:
	CMapViewMoveTracker(CMapManager *manager,QWidget *parent=0, const char *name=0);
	virtual ~CMapViewMoveTracker();

	/** This method is called when an element is added */
	virtual void addedElement(CMapElement *element);
	/** This method is called when an element is deleted */
	virtual void deletedElement(CMapLevel *deletedFromLevel);
	/** This method is called when an element is changed */
	virtual void changedElement(CMapElement *element);
	/** This method is called when a map level is changed */
	virtual void changedLevel(CMapLevel *level);
	/** Tell this map widget to display a different zone */
	virtual void showPosition(QPoint pos,CMapLevel *level,bool centerView=true);

	/** Used to find the width of the widget */
	virtual int getWidth(void);
	/** Used to find the height of the widget */
	virtual int getHeight(void);

	/** Used to set the follow mode */
	virtual void setFollowMode(bool follow);
	/** Get the state of the follow combo box */
	virtual bool getFollowMode(void);

	/** Used to let the map manager know if it should register the focus of this widget */
	virtual bool acceptFocus(void);

	/** Used to set the view to active */
	virtual void setActive(bool active);

	/** This view does not support a view area */
	virtual bool getSupportsViewArea(void)                               { return false; }

	virtual void redraw();
	/** This is used to generate the conents of the view */
	void generateContents(void);
	/** This method is called when the current position of the player changes */
	virtual void playerPositionChanged(CMapRoom *room);

protected:	
	/** Used to find out if a element is visible on the widget */
	virtual void paintEvent(QPaintEvent *);
	virtual void resizeEvent(QResizeEvent *);
	//virtual void checkSize(QPoint pos);

private:
	bool m_needsCreate;
	/** The double buffer */
	QPixmap *m_buffer;
	/** This postion to center the view onto */
	QPoint m_pos;

};

#endif
