/***************************************************************************
                               cmappluginspeedwalk.cpp
                             -------------------
    begin                : Mon Aug 6 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmappluginspeedwalk.h"

#include <qiconset.h>

#include <kstddirs.h>
#include <kaction.h>
#include <klocale.h>

#include "cmapspeedwalkfactory.h"
#include "dlgmapspeedwalkcontainter.h"
#include "dlgmapspeedwalklist.h"
#include "cmapcmdspeedwalkadd.h"
#include "cmapcmdspeedwalkremove.h"

#include "../../cmapmanager.h"
#include "../../cmaproom.h"

#include <kmuddockwidget.h>
#include <kmuddebug.h>
#include <kmudviewmanager.h>

CMapPluginSpeedwalk::CMapPluginSpeedwalk(CMapManager *manager,QObject *parent, const char *name ) : CMapPluginBase(manager,parent,name)
{
	// setting instance name
	setInstance(CMapSpeedwalkFactory::instance());

	setXMLFile (locate("data", "kmudmapper/plugins/kmudmapper_speedwalk.rc"));

	m_showAction = new KAction ( i18n("Show Speedwalk List"),0,this,SLOT(slotViewSpeedwalkList()),actionCollection(), "viewSpeedwalkList");
	m_addAction = new KAction(i18n("&Add to speed walk list"),0, this,SLOT(slotRoomAddToSpeedwalk()), actionCollection(),"roomAddToSpeedwalk");
	
	m_speedwalkRoomList.setAutoDelete(false);
	m_speedwalkList = NULL;

	m_showAction->setEnabled(false);
}

CMapPluginSpeedwalk::~CMapPluginSpeedwalk()
{
}

void CMapPluginSpeedwalk::saveConfigOptions(void)
{
	KConfig *config = instance()->config();

	config->setGroup("SPEEDWALK");
	config->writeEntry("Catogrize",m_speedwalkCatogrize);
}

void CMapPluginSpeedwalk::loadConfigOptions(void)
{
	KConfig *config = instance()->config();

	config->setGroup("SPEEDWALK");
	m_speedwalkCatogrize = config->readNumEntry("Catogrize",0);
}

int CMapPluginSpeedwalk::getSpeedwalkCatogrize()
{
	return m_speedwalkCatogrize;
}

void CMapPluginSpeedwalk::setSpeedwalkCatogrize(int catogry)
{
	m_speedwalkCatogrize = catogry;
}

QPtrList<CMapRoom> *CMapPluginSpeedwalk::getSpeedwalkRoomList()
{
	return &m_speedwalkRoomList;
}

KmudDockWidget* CMapPluginSpeedwalk::showSpeedwalkList()
{
	if (!m_speedwalkList)
	{
		m_speedwalkList = new DlgMapSpeedwalkContainter(this,mapManager->widget(),"SpeedwalkList");
		m_speedwalkList->show();
	}

	return m_speedwalkList;
}

void CMapPluginSpeedwalk::hideSpeedwalkList()
{
	if (m_speedwalkList)
	{
		delete m_speedwalkList;
		m_speedwalkList=NULL;
	}
}

/** Used to add a room to the speed walk list */
void CMapPluginSpeedwalk::addSpeedwalkRoom(CMapRoom *room)
{
	CMapCmdSpeedwalkAdd *cmd = new CMapCmdSpeedwalkAdd(this,room);
	mapManager->addCommand(cmd);

}

/** Used to delete a room from the speed walk list. */
void CMapPluginSpeedwalk::delSpeedwalkRoom(CMapRoom *room)
{
	CMapCmdSpeedwalkRemove *cmd = new CMapCmdSpeedwalkRemove(this);
	cmd->addRoom(room);
	mapManager->addCommand(cmd);
}

/** Used to add a room to the speed walk list */
void CMapPluginSpeedwalk::addSpeedwalkRoomNoCmd(CMapRoom *room,bool update)
{
	if (m_speedwalkRoomList.findRef(room)==-1)
	{
		m_speedwalkRoomList.append(room);
	}

	if (m_speedwalkList && update)
	{
		m_speedwalkList->getSpeedwalkList()->updateSpeedwalkList(room);
	}
}

/** This is called when the character or mud profiles change */
void CMapPluginSpeedwalk::profileChanged(void)
{
	m_showAction->setEnabled(true);
}

/** This method is called to recreate the speedwalk list */
void CMapPluginSpeedwalk::updateSpeedwalkList()
{
	if (m_speedwalkList)
	{
		m_speedwalkList->getSpeedwalkList()->updateSpeedwalkList();
	}
}

/** Used to delete a room from the speed walk list. */
void CMapPluginSpeedwalk::delSpeedwalkRoomNoCmd(CMapRoom *room,bool update)
{
	m_speedwalkRoomList.remove(room);

	if (m_speedwalkList && update)
		m_speedwalkList->getSpeedwalkList()->updateSpeedwalkList();	
}

/** Used to add the room under the point to the speedwalk list */
void CMapPluginSpeedwalk::slotRoomAddToSpeedwalk()
{
	addSpeedwalkRoom((CMapRoom *)mapManager->getSelectedElement());
}

void CMapPluginSpeedwalk::slotViewSpeedwalkList()
{
	KmudDockWidget *list = showSpeedwalkList();
	mapManager->getViewManager()->addWidget(list,"Kmud/Mapper/SpeedwalkList",KmudViewManager::DockLeft,"Kmud/Mapper");
}

/** This is called before a element menu is openend */
void CMapPluginSpeedwalk::beforeOpenElementMenu(CMapElement *element)
{
	m_addAction->setEnabled(m_speedwalkRoomList.findRef((CMapRoom*)element)==-1);
}

/** This is called before a element is deleted */
void CMapPluginSpeedwalk::beforeElementDeleted(CMapElement *element)
{
	if (element->getElementType()==ROOM)
	{
		delSpeedwalkRoom((CMapRoom *)element);
	}
}

/** This is called when a element is changed */
void CMapPluginSpeedwalk::elementChanged(CMapElement *element)
{
	if (element->getElementType()==ZONE)
	{
		if (m_speedwalkList)
		{
			CMapZone *zone = (CMapZone *)element;
			for (CMapRoom *room = m_speedwalkRoomList.first(); room !=0; room = m_speedwalkRoomList.next())
			{
				if (room->getZone()==zone)
				{
					m_speedwalkList->getSpeedwalkList()->updateSpeedwalkList();
					break;
				}
			}
		}
	}

	if (element->getElementType()==ROOM)
	{
		if (m_speedwalkList)
		{
			if (m_speedwalkRoomList.find((CMapRoom *)element))
				m_speedwalkList->getSpeedwalkList()->updateSpeedwalkList((CMapRoom *)element);
		}
	}
}


void CMapPluginSpeedwalk::mapErased(void)
{
	m_speedwalkRoomList.clear();

	if (m_speedwalkList)
	{
		m_speedwalkList->getSpeedwalkList()->updateSpeedwalkList();
	}
}

/** This method is used to get a list of new properties for a element
  * It will usally be called when saving map data to file
  * @param element The element being saved
  * @param properties When method exits this should contain the new properties
  */
void CMapPluginSpeedwalk::saveElementProperties(CMapElement *element,KMemConfig *properties)
{
	if (element->getElementType()==ROOM)
	{
		if (m_speedwalkRoomList.contains((CMapRoom *)element))
		{
			properties->writeEntry("SpeedwalkRoom","Yes");
		}
	}
}

/** This method is used to update an element with the properties load from a file
  * It will usally be called when loading map data to file
  * @param element The element being loaded
  * @param properties The properties being loaded from the file
  */
void CMapPluginSpeedwalk::loadElementProperties(CMapElement *element,KMemConfig *properties)
{
	if (element->getElementType()==ROOM && properties->hasKey("SpeedwalkRoom"))
	{
		addSpeedwalkRoomNoCmd((CMapRoom *)element);
	}
}
