/***************************************************************************
                               cmapoverview.cpp
                             -------------------
    begin                : Fri May 11 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmapoverview.h"

#include <qwidget.h>
#include <qevent.h>

#include "../../../cmapmanager.h"
#include "../../../cmaplevel.h"

CMapOverview::CMapOverview(CMapManager *manager,QWidget *parent, const char *name ) : CMapViewBase(manager,parent,name)
{
	buffer = NULL;
	setBackgroundMode(NoBackground);

	maxSize = QSize(0,0);
	checkSize(QPoint(300,300));

	needsCreate = true;
}

CMapOverview::~CMapOverview()
{
	if (buffer)
		delete buffer;
}

/** This method is called when an element is added */
void CMapOverview::addedElement(CMapElement *element)
{
	if (isElementVisibale(element))
	{
		checkSize(element->getHighPos());
		redraw();
	}
}

/** This method is called when an element is deleted */
void CMapOverview::deletedElement(CMapLevel *deletedFromLevel)
{
	QPaintEvent tmpEvent();

	CMapLevel *upperLevel = getCurrentlyViewedLevel()->getNextLevel();
	CMapLevel *lowerLevel = getCurrentlyViewedLevel()->getPrevLevel();


	if (deletedFromLevel == getCurrentlyViewedLevel())
	{
		redraw();
	}

	if (upperLevel  && mapManager->getMapData()->showUpperLevel)
		if (deletedFromLevel == upperLevel)
			redraw();

	if (lowerLevel  && mapManager->getMapData()->showLowerLevel)
		if (deletedFromLevel == lowerLevel)
			redraw();
}

/** This method is called when an element is changed */
void CMapOverview::changedElement(CMapElement *element)
{
	if (isElementVisibale(element))
	{
		checkSize(element->getHighPos());
		redraw();
	}
}

/** This method is called when a map level is changed */
void CMapOverview::changedLevel(CMapLevel *level)
{
	if (isLevelVisibale(level))
	{
		int maxx=0,maxy=0;
		for (CMapElement *element=level->getFirstElement();element!=0;element=level->getNextElement())
		{
			if (element->getHighX()>maxx)
				maxx =element->getHighX();

			if (element->getHighY()>maxx)
				maxx = element->getHighY();

		
		}
		redraw();
		checkSize(QPoint(maxx,maxy));
	}
}

/** Tell this map widget to display a different zone */
void CMapOverview::showPosition(QPoint,CMapLevel *level,bool)
{
	setLevel(level);

	CMapLevel *upperLevel = level->getNextLevel();
	CMapLevel *lowerLevel = level->getPrevLevel();

	QPoint size(0,0);

	// Calc the size the widget should be
	// FIXME_jp : Add lower level elements
	for (CMapElement *element =level->getFirstElement();element!=NULL;element=level->getNextElement())
	{
		if (element->getHighX()>size.x()) size.setX(element->getHighX());
		if (element->getHighY()>size.y()) size.setY(element->getHighY());
	}

	// FIXME_jp: Do a check to see if this needs to be done
	if (upperLevel  && mapManager->getMapData()->showUpperLevel)
	{
		for (CMapElement *element =upperLevel->getFirstElement();element!=NULL;element=upperLevel->getNextElement())
		{
			if (element->getHighX()>size.x()) size.setX(element->getHighX());
			if (element->getHighY()>size.y()) size.setY(element->getHighY());
		}

	}
	if (lowerLevel  && mapManager->getMapData()->showLowerLevel)
	{
		for (CMapElement *element =lowerLevel->getFirstElement();element!=NULL;element=lowerLevel->getNextElement())
		{
			if (element->getHighX()>size.x()) size.setX(element->getHighX());
			if (element->getHighY()>size.y()) size.setY(element->getHighY());
		}

	}

	// Resize the widget
	maxSize = QSize(0,0);
	checkSize(size);

	redraw();
}

/** Used to calculate the correct size for the widget */
void CMapOverview::checkSize(QPoint pos)
{
	if (pos.x() + (mapManager->getMapData()->gridSize.width() * 3) > maxSize.width())
		maxSize.setWidth(pos.x()+ (mapManager->getMapData()->gridSize.width() * 3));
	if (pos.y() + (mapManager->getMapData()->gridSize.height() * 3) > maxSize.height())
		maxSize.setHeight(pos.y()+ (mapManager->getMapData()->gridSize.height() * 3));
}

/** Used to find the width of the widget */
int CMapOverview::getWidth(void)
{
	return width();
}

/** Used to find the height of the widget */
int CMapOverview::getHeight(void)
{
	return height();
}

/** Used to set the follow mode */
void CMapOverview::setFollowMode(bool)
{
	// DO NOTHING
}

/** Get the state of the follow combo box */
bool CMapOverview::getFollowMode(void)
{
	return false;
}

/** Used to let the map manager know if it should register the focus of this widget */
bool CMapOverview::acceptFocus(void)
{
	return false;
}

/** Used to set the view to active */
void CMapOverview::setActive(bool)
{
	// DO NOTHING
}

/** This is used to generate the conents of the view */
void CMapOverview::generateContents(void)
{
	CMapViewBase *view = mapManager->getActiveView();
	QRect drawArea(0,0,width(), height());

	// Create double buffer
	if (buffer && (buffer->size() != drawArea.size()))
	{
		delete buffer;
		buffer = NULL;
	}
	
	if (!buffer)
	{
		buffer = new QPixmap(drawArea.size());
	}

	if (needsCreate)
	{
		QPainter p;
    	
		p.begin(buffer);

		if (view)
		{
			// Draw the background
			if (getCurrentlyViewedZone()->getUseDefaultBackground())
			{
				p.fillRect(0,0,width(),height(),mapManager->getMapData()->backgroundColor);
			}
			else
			{
				p.fillRect(0,0,width(),height(),getCurrentlyViewedZone()->getBackgroundColor());
			}
	
			// Scale so all elements fit in draw area
			double xscale = ((double)width())/((double)maxSize.width());
			double yscale = ((double)height())/((double)maxSize.height());
	
			double scale;
			if (xscale < yscale)
			{
				scale = xscale;
			}
			else
			{
				scale = yscale;
			}

			if (scale>1)
				scale = 1;
	
			//p.translate((maxSize.width()/2)-(width()/2),(maxSize.height()/2)-(height()/2) );
			p.scale(scale,scale);

			p.setPen(black);
			p.drawRect(0,0,maxSize.width(),maxSize.height());

			// Draw Elements
			drawElements(&p);
	
			if (view ->getSupportsViewArea())
			{
				// FIXME_jp: Draw visuble area indicator
				// p.setPen(Qt::red);
				// p.drawRect(view->getViewArea());
			}
		}
		else
		{
			p.fillRect(0,0,width(),height(),mapManager->getMapData()->backgroundColor);
		}

		p.end();
		needsCreate = false;
	}
}


void CMapOverview::paintEvent(QPaintEvent *)
{
	generateContents();

	bitBlt(this, 0, 0, buffer);
}

void CMapOverview::redraw(void)
{
	update();
	needsCreate = true;
}

void CMapOverview::resizeEvent(QResizeEvent *)
{
	needsCreate = true;
}

void CMapOverview::viewChanged(CMapLevel *level)
{
	((CMapViewBase *)this)->showPosition(level);
}
